"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.AppBase = exports.DEFAULT_RATE_LIMIT_MAX_REQ_PER_WINDOW = exports.DEFAULT_RATE_LIMIT_WINDOW = exports.express = void 0;
const body_parser_1 = __importDefault(require("body-parser"));
const cm_libs_context_aware_error_1 = require("cm-libs-context-aware-error");
const cm_libs_context_aware_logger_1 = require("cm-libs-context-aware-logger");
const cm_libs_utils_1 = require("cm-libs-utils");
const compression_1 = __importDefault(require("compression"));
const cors_1 = __importDefault(require("cors"));
const express_1 = __importDefault(require("express"));
exports.express = express_1.default;
const express_rate_limit_1 = __importDefault(require("express-rate-limit"));
const helmet_1 = __importDefault(require("helmet"));
const http_1 = require("http");
const multer_1 = __importDefault(require("multer"));
const ws_1 = require("ws");
const common_1 = require("./common");
const correlation_1 = __importDefault(require("./common/misc/correlation"));
const app_seed_1 = require("./app-seed");
const upload = (0, multer_1.default)();
const ENABLE_CORRELATOR = common_1.ConfigBase.get('ENABLE_CORRELATOR', (0, cm_libs_utils_1.allowNullOrEmpty)(cm_libs_utils_1.checkBoolean, false));
exports.DEFAULT_RATE_LIMIT_WINDOW = 15 * 60000; // 15 minutes
exports.DEFAULT_RATE_LIMIT_MAX_REQ_PER_WINDOW = 100;
class AppBase extends app_seed_1.AppSeed {
    app;
    listener;
    httpServer;
    wsServer;
    apolloServer;
    correlator;
    constructor(name, version) {
        const thisCorrelator = ENABLE_CORRELATOR ? correlation_1.default : undefined;
        super(name, version, thisCorrelator);
        this.correlator = thisCorrelator;
    }
    async init(params) {
        params ||= {};
        // Create an Express app and HTTP server; we will attach both the WebSocket
        // server and the ApolloServer to this HTTP server.
        this.app = (0, express_1.default)();
        this.httpServer = (0, http_1.createServer)(this.app);
        if (ENABLE_CORRELATOR)
            this.app.use(this.correlator());
        this.app.set('x-powered-by', false);
        // Setup base middlware
        this.setupBaseMiddleware(params);
        // Setup general context middleware
        this.app.use(common_1.contextMiddleware);
        // Setup auth components
        this.setupAuthComponents(params);
        // Initialize variables from parameters
        this.serviceIdToken = params.serviceIdToken;
        this.serviceOrgId = params.orgId;
        this.serviceAppDomain = params.appDomain;
        this.serviceProductId = params.productId;
        this.serviceAccessInfo = params.serviceAccessInfo;
        // Setup base components
        this.setupBaseComponents(params);
        // Setup logger middleware
        this.app.use(common_1.logResMiddleware);
        // Setup debug logger middleware
        if (this.devMode && !params.options?.disableDebugTrace) {
            this.app.use(common_1.dumpResMiddleware);
            this.app.use(common_1.dumpResErrorMiddleware);
        }
        return this;
    }
    setupBaseMiddleware(params) {
        const { origins, rateLimitHandler, options, multipartFormData, enablePubSub } = params;
        if (!this.devMode) {
            this.app.use((0, helmet_1.default)());
        }
        // Apply the rate limiting middleware to all requests
        if (rateLimitHandler) {
            this.app.use((0, express_rate_limit_1.default)({
                windowMs: 0 < options?.RATE_LIMIT_WINDOW ? options?.RATE_LIMIT_WINDOW : exports.DEFAULT_RATE_LIMIT_WINDOW,
                max: 0 < options?.RATE_LIMIT_MAX_REQ_PER_WINDOW ? options?.RATE_LIMIT_MAX_REQ_PER_WINDOW : exports.DEFAULT_RATE_LIMIT_MAX_REQ_PER_WINDOW,
                standardHeaders: true,
                legacyHeaders: false,
                handler: rateLimitHandler
            }));
        }
        if (enablePubSub) {
            // Create our WebSocket server using the HTTP server we just set up.
            this.wsServer = new ws_1.WebSocketServer({
                server: this.httpServer,
                path: '/graphql'
            });
        }
        // Setup default CORS
        this.app.use((0, cors_1.default)({
            origin: origins?.split(',') || ['*'],
            allowedHeaders: [
                'origin',
                'x-requested-with',
                'content-type',
                'accept',
                'lang',
                'authorization',
                'accept-version',
                'content-language',
                'app-id',
                'location',
                'x-org-id',
                'x-app-domain',
                'x-product-id',
                'x-role-type',
                'x-has-ability',
                'x-metadata'
            ],
            preflightContinue: false,
            ...options
        }));
        if (options?.bodyParser) {
            // Setup default body size to 3MB
            this.app.use(options.bodyParser, body_parser_1.default.json({ limit: 3 * 1024 * 1024 }));
            // Parse incoming request body and append data to `req.body`
            this.app.use(options.bodyParser, body_parser_1.default.urlencoded({ extended: true }));
        }
        else {
            // Setup default body size to 3MB
            this.app.use(body_parser_1.default.json({ limit: 3 * 1024 * 1024 }));
            // Parse incoming request body and append data to `req.body`
            this.app.use(body_parser_1.default.urlencoded({ extended: true }));
        }
        // for parsing multipart/form-data
        if (multipartFormData) {
            this.app.use(upload.any());
            this.app.use(express_1.default.static('public'));
        }
        // Setup response timer
        this.app.use((req, res, next) => {
            req['start'] = (0, cm_libs_utils_1.timeNow)();
            return next();
        });
        // compress all responses
        if (options?.compression) {
            if (typeof options.compression === 'object') {
                this.app.use((0, compression_1.default)(options.compression));
            }
            else {
                this.app.use((0, compression_1.default)());
            }
        }
    }
    setupBaseComponents(params) {
        const options = (params.options ||= {});
        // Setup error handler
        const ERROR_URL = options.errorUrl || common_1.ConfigBase.get('ERROR_URL', (0, cm_libs_utils_1.allowNullOrEmpty)(cm_libs_utils_1.checkString));
        if (ERROR_URL) {
            common_1.ErrorHandler.init(ERROR_URL, this.serviceAccessInfo);
        }
        // Setup schema registry
        const SCHEMA_REGISTRY_URL = common_1.ConfigBase.get('SCHEMA_REGISTRY_URL', (0, cm_libs_utils_1.allowNullOrEmpty)(cm_libs_utils_1.checkString));
        const SCHEMA_REGISTRY_OVERRIDES = common_1.ConfigBase.get('SCHEMA_REGISTRY_OVERRIDES', (0, cm_libs_utils_1.allowNullOrEmpty)(cm_libs_utils_1.checkString));
        if (SCHEMA_REGISTRY_URL) {
            common_1.Registry.init(SCHEMA_REGISTRY_URL, SCHEMA_REGISTRY_OVERRIDES ? JSON.parse(SCHEMA_REGISTRY_OVERRIDES) : undefined);
        }
    }
    setupAuthComponents(params) {
        const options = (params.options ||= {});
        // Exclude these services that implemented verification (e.g. identity service)
        if (!options.identityUrl && !options.fnBearerVerify) {
            options.identityUrl = common_1.ConfigBase.get('IDENTITY_URL', (0, cm_libs_utils_1.notAllowNullOrEmpty)(cm_libs_utils_1.checkString));
        }
        if (undefined === options.identityCacheMax) {
            options.identityCacheMax = common_1.ConfigBase.get('IDENTITY_CACHE_MAX', (0, cm_libs_utils_1.allowNullOrEmpty)(cm_libs_utils_1.checkNumber));
        }
        if (undefined === options.identityCacheDuration) {
            options.identityCacheDuration = common_1.ConfigBase.get('IDENTITY_CACHE_DURATION', (0, cm_libs_utils_1.allowNullOrEmpty)(cm_libs_utils_1.checkNumber));
        }
        // Setup authentication bearer token middleware
        if (options.identityUrl || options.fnBearerVerify) {
            if (!params.serviceIdToken) {
                params.serviceIdToken = common_1.ConfigBase.get('SERVICE_ID_TOKEN', (0, cm_libs_utils_1.notAllowNullOrEmpty)(cm_libs_utils_1.checkString));
            }
            if (undefined === params.orgId) {
                params.orgId = common_1.ConfigBase.get('SERVICE_ORG_ID', (0, cm_libs_utils_1.notAllowNullOrEmpty)(cm_libs_utils_1.checkNumber));
            }
            if (undefined === params.appDomain) {
                params.appDomain = common_1.ConfigBase.get('SERVICE_APP_DOMAIN', (value) => {
                    const { error, value: appDomain } = common_1._appDomain.validate(value);
                    if (error) {
                        (0, cm_libs_context_aware_error_1.throwError)(cm_libs_context_aware_error_1.errors.INVALID_FORMAT, error.message, {
                            function: 'setupBaseComponents',
                            details: error.details,
                            text: { input: error.details?.[0]?.context?.peers || error.details?.[0]?.context?.key }
                        });
                    }
                    return appDomain;
                });
            }
            if (undefined === params.productId) {
                params.productId = common_1.ConfigBase.get('SERVICE_PRODUCT_ID', (0, cm_libs_utils_1.notAllowNullOrEmpty)(cm_libs_utils_1.checkNumber));
            }
            common_1.AuthBearer.init(options?.fnBearerVerify || options.identityUrl, params.serviceIdToken, params.orgId, params.appDomain, params.productId, options.identityCacheMax, options.identityCacheDuration);
            this.app.use(common_1.authBearerTokenMiddleware);
        }
        // Setup model middleware
        const isEnabled = params.options?.disableModelRoleAbilityLog ? false : this.devMode;
        const disableLogMessage = undefined === params.options?.disableModelRoleAbilityLogMessage ? true : params.options?.disableModelRoleAbilityLogMessage;
        common_1.BaseModelMiddlewareRoleAbilityLog.init(isEnabled, params.serviceIdToken, options.identityUrl, disableLogMessage);
        // Setup authentication basic token middleware
        if (options.fnVerify) {
            common_1.AuthBasic.init(options.fnVerify);
            this.app.use(common_1.authBasicTokenMiddleware);
        }
    }
    async term() {
        if (this.apolloServer) {
            await this.apolloServer.stop();
            this.apolloServer = undefined;
        }
        else {
            // Handle non-apollo server shutdown
            if (this.listener) {
                await new Promise((resolve, reject) => {
                    try {
                        this.listener.close((error) => {
                            if (error) {
                                return reject(error);
                            }
                            this.listener = undefined;
                            return resolve();
                        });
                    }
                    catch (error) {
                        /* istanbul ignore next */
                        return reject(error);
                    }
                });
            }
        }
        await super.term();
    }
    async run(port, hostname) {
        // Setup default endpoints
        this.app.get('/', this.handler(this, this.onRoot, undefined, false));
        this.app.get('/liveness', this.handler(this, this.onLiveness, undefined, false));
        this.app.get('/readiness', this.handler(this, this.onReadiness, undefined, false));
        // Setup API documentation endpoint
        if (this.devMode) {
            this.app.use('/apidoc', express_1.default.static('apidoc', { index: ['index.html'] }));
        }
        // Setup 404 handler middleware
        this.app.use(common_1.finalMiddleware); // NOTE: Make sure this handler is place at the last of the router stack
        // Setup error middleware
        this.app.use(common_1.errorMiddleware);
        // Modified server startup
        return new Promise((resolve, reject) => {
            try {
                port = port || 8080;
                this.listener = this.httpServer?.listen(port, hostname || '0.0.0.0', () => {
                    return resolve(`http://${hostname || '*'}:${port}`);
                });
            }
            catch (error) {
                /* istanbul ignore next */
                return reject(error);
            }
        });
    }
    handler(thisObject, func, authDirective, logging = true, raw = false) {
        return async (req, res, next) => {
            try {
                if (logging) {
                    if (this.devMode) {
                        (0, cm_libs_context_aware_logger_1.log)(cm_libs_context_aware_logger_1.Level.CRITICAL, () => {
                            const headers = JSON.stringify(req.headers, null, 2);
                            const body = JSON.stringify(req.body, null, 2);
                            return `TRACE REQ: ${req.method} ${req.url} HTTP/${req.httpVersion} @${(0, common_1.getIp)(req)}\nHEADERS: ${headers}\nBODY: ${body}`;
                        });
                    }
                    else {
                        (0, cm_libs_context_aware_logger_1.log)(cm_libs_context_aware_logger_1.Level.NORMAL, () => `REQ: ${req.method} ${req.url} HTTP/${req.httpVersion} @${(0, common_1.getIp)(req)}`);
                    }
                }
                else {
                    req['logExcluded'] = true;
                }
                if (authDirective) {
                    const accessToken = await (0, common_1.auth)(req, authDirective);
                    if (authDirective.hasAbility) {
                        try {
                            (0, common_1.hasAbility)(accessToken, authDirective.hasAbility);
                        }
                        catch (error) {
                            const localeCode = accessToken?.info?.headers?.localeCode || common_1.LocaleCode.EN_US;
                            const { code, message } = await common_1.ErrorHandler.translate(error, error.statusCode, localeCode);
                            error.code = code;
                            error.message = message;
                            (0, cm_libs_context_aware_error_1.throwFromError)(error, cm_libs_context_aware_error_1.errors.FORBIDDEN, { spec: authDirective.hasAbility });
                        }
                    }
                }
                const data = await func.apply(thisObject, [req, res, next]);
                if (undefined !== data) {
                    (0, common_1.goodRes)(res, data, undefined, raw);
                }
            }
            catch (error) {
                next(error);
            }
        };
    }
    fileHandler(cachePath, maxFileSize, fileFieldName) {
        return async (req, res, next) => {
            try {
                const options = {
                    dest: cachePath,
                    limits: {
                        fileSize: maxFileSize
                    }
                };
                const upload = (0, multer_1.default)(options);
                return upload.single(fileFieldName || 'file')(req, res, next);
            }
            catch (error) {
                /* istanbul ignore next */
                return next(error);
            }
        };
    }
    /**
     * @api {get} /liveness Liveness health check
     * @apiVersion 1.0.0
     * @apiName onLiveness
     * @apiGroup HealthCheck
     * @apiPermission none
     *
     * @apiDescription Service liveness health check.
     *
     * @apiSuccessExample Success-Response:
     *     HTTP/1.1 200 OK
     *
     * @apiErrorExample Error-Response:
     *     HTTP/1.1 500 Service is offline
     */
    async onLiveness(req, res) {
        res.status(this.liveness ? 200 : 500).send();
    }
    /**
     * @api {get} /readiness Readiness health check
     * @apiVersion 1.0.0
     * @apiName onReadiness
     * @apiGroup HealthCheck
     * @apiPermission none
     *
     * @apiDescription Service readiness health check.
     *
     * @apiSuccessExample Success-Response:
     *     HTTP/1.1 200 OK
     *
     * @apiErrorExample Error-Response:
     *     HTTP/1.1 429 Service is not ready
     */
    async onReadiness(req, res) {
        res.status(this.readiness ? 200 : 429).send();
    }
    /**
     * @api {get} / Service information
     * @apiVersion 1.0.0
     * @apiName onRoot
     * @apiGroup HealthCheck
     * @apiPermission none
     *
     * @apiDescription Service information.
     *
     * @apiSuccess {String} message Service information
     *
     * @apiSuccessExample Success-Response:
     *     HTTP/1.1 200 OK
     *     {
     *       "message": "Welcome to the coolest API on earth - ServiceName:1.0.0"
     *     }
     */
    onRoot(req, res) {
        res.json({
            message: `Welcome to the coolest API on earth - ${this.name}:${this.version}, remoteIp: ${(0, common_1.getIp)(req)}!`
        });
    }
}
exports.AppBase = AppBase;
//# sourceMappingURL=app-base.js.map