"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.AuthBaseToken = void 0;
const jsonwebtoken_1 = __importDefault(require("jsonwebtoken"));
const cm_libs_context_aware_error_1 = require("cm-libs-context-aware-error");
const moment_1 = __importDefault(require("moment"));
class AuthBaseToken {
    secret;
    schema;
    signOptions;
    kind;
    extendedKind;
    constructor(secret, expiresIn, schema, kind, extendedKind, algorithm) {
        this.secret = secret || (0, cm_libs_context_aware_error_1.throwError)(cm_libs_context_aware_error_1.errors.PARAMETER_MISSING, 'Auth token secret is missing');
        this.schema = schema;
        this.kind = kind;
        this.extendedKind = extendedKind;
        this.signOptions = {
            expiresIn
        };
        if (undefined !== algorithm) {
            this.signOptions.algorithm = algorithm;
        }
    }
    sign(payload, expiresIn, notBefore) {
        const claim = AuthBaseToken.validate(payload, this.schema);
        claim['kind'] = undefined === payload['extended'] ? this.kind : this.extendedKind;
        const signOptions = { ...this.signOptions };
        if (expiresIn) {
            if (expiresIn instanceof Date) {
                const diff = (0, moment_1.default)(expiresIn).diff(new Date(), 'seconds');
                if (0 < diff) {
                    signOptions.expiresIn = diff;
                }
            }
            else {
                signOptions.expiresIn = expiresIn;
            }
        }
        if (notBefore) {
            if (notBefore instanceof Date) {
                signOptions.notBefore = (0, moment_1.default)(notBefore).diff(new Date(), 'seconds');
            }
            else {
                signOptions.notBefore = notBefore;
            }
        }
        return jsonwebtoken_1.default.sign(claim, this.secret, signOptions);
    }
    verify(token) {
        let claim;
        try {
            claim = jsonwebtoken_1.default.verify(token, this.secret);
        }
        catch (error) {
            switch (error?.name) {
                case 'TokenExpiredError':
                    (0, cm_libs_context_aware_error_1.throwFromError)(error, cm_libs_context_aware_error_1.errors.EXPIRED, { token }, false);
                case 'NotBeforeError':
                    (0, cm_libs_context_aware_error_1.throwFromError)(error, cm_libs_context_aware_error_1.errors.NOT_READY, { token }, false);
            }
            (0, cm_libs_context_aware_error_1.throwFromError)(error, cm_libs_context_aware_error_1.errors.UNAUTHORISED, { token }, false);
        }
        const expectedKind = undefined === claim?.['extended'] ? this.kind : this.extendedKind;
        /* istanbul ignore next */
        if (expectedKind !== claim?.['kind']) {
            (0, cm_libs_context_aware_error_1.throwError)(cm_libs_context_aware_error_1.errors.MISMATCHED, 'Token kind is mismatched', { claim, expected: expectedKind });
        }
        return AuthBaseToken.validate(claim, this.schema);
    }
    static baseDecode(token, kind, schema) {
        let claim;
        try {
            claim = jsonwebtoken_1.default.decode(token);
        }
        catch (error) {
            switch (error?.name) {
                case 'TokenExpiredError':
                    (0, cm_libs_context_aware_error_1.throwFromError)(error, cm_libs_context_aware_error_1.errors.EXPIRED, { token }, false);
                case 'NotBeforeError':
                    (0, cm_libs_context_aware_error_1.throwFromError)(error, cm_libs_context_aware_error_1.errors.NOT_READY, { token }, false);
            }
            (0, cm_libs_context_aware_error_1.throwFromError)(error, cm_libs_context_aware_error_1.errors.UNAUTHORISED, { token }, false);
        }
        /* istanbul ignore next */
        if (undefined !== kind && kind !== claim?.['kind']) {
            (0, cm_libs_context_aware_error_1.throwError)(cm_libs_context_aware_error_1.errors.MISMATCHED, 'Token kind is mismatched', { claim, expected: kind });
        }
        return this.validate(claim, schema);
    }
    static validate(claim, schema) {
        const { error, value: validatedClaim } = schema.validate(claim, { stripUnknown: true });
        /* istanbul ignore next */
        if (error) {
            (0, cm_libs_context_aware_error_1.throwError)(cm_libs_context_aware_error_1.errors.INVALID_FORMAT, error.message, {
                function: 'validate',
                details: error.details,
                text: { input: error.details?.[0]?.context?.peers || error.details?.[0]?.context?.key }
            });
        }
        return validatedClaim;
    }
}
exports.AuthBaseToken = AuthBaseToken;
//# sourceMappingURL=auth-base-token.js.map