"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.AuthBearer = void 0;
const joi_1 = __importDefault(require("joi"));
const crypto_1 = __importDefault(require("crypto"));
const graphql_1 = require("graphql");
const graphql_tag_1 = require("graphql-tag");
const cm_libs_context_aware_error_1 = require("cm-libs-context-aware-error");
const intercall_1 = require("../intercall");
const auth_type_1 = require("./auth-type");
const auth_type_verify_1 = require("./auth-type-verify");
const auth_id_token_1 = require("./auth-id-token");
const type_1 = require("../type");
class AuthBearer {
    static self;
    identityUrl;
    fnVerify;
    serviceIdToken;
    filter;
    caches = {};
    cacheMax;
    cacheDuration;
    constructor(urlOrfnVerify) {
        if ('string' === typeof urlOrfnVerify) {
            this.identityUrl = urlOrfnVerify;
        }
        else {
            this.fnVerify = urlOrfnVerify;
        }
        if (!this.identityUrl && !this.fnVerify) {
            (0, cm_libs_context_aware_error_1.throwError)(cm_libs_context_aware_error_1.errors.PARAMETER_MISSING, 'Identity URL is missing', { urlOrfnVerify });
        }
    }
    static init(urlOrfnVerify, serviceIdToken, orgId, appDomain, productId, cacheMax, cacheDuration) {
        const self = new AuthBearer(urlOrfnVerify);
        // Retrieve service access info from the identity service
        const claim = auth_id_token_1.AuthIdToken.decode(serviceIdToken);
        // Service Id Token claim must be matched with the configured values
        if ((claim.orgId && claim.orgId !== orgId) || (claim.appDomain && claim.appDomain !== appDomain) || (claim.productId && claim.productId !== productId)) {
            (0, cm_libs_context_aware_error_1.throwError)(cm_libs_context_aware_error_1.errors.MISMATCHED, 'Service Id Token claim is mismatched', { claim, expected: { orgId, appDomain, productId } });
        }
        self.filter = {
            orgId,
            appDomain,
            productId
        };
        self.serviceIdToken = serviceIdToken;
        self.cacheMax = 0 < cacheMax ? cacheMax : 0;
        self.cacheDuration = 0 < cacheDuration ? cacheDuration : 0;
        this.self = self;
        return self;
    }
    static term() {
        if (this.self && 0 < this.self.cacheDuration) {
            Object.keys(this.self.caches).forEach((key) => {
                const cache = this.self.caches[key];
                clearTimeout(cache.expiryTimer);
                delete this.self.caches[key];
            });
        }
    }
    static get instance() {
        return this.self || (0, cm_libs_context_aware_error_1.throwError)(cm_libs_context_aware_error_1.errors.NOT_INITIALIZED, 'AuthBearer initialization is required');
    }
    static get isEnabled() {
        return undefined !== this.self;
    }
    get serviceIdTokenInfo() {
        return {
            token: this.serviceIdToken,
            orgId: this.filter.orgId,
            appDomain: this.filter.appDomain,
            productId: this.filter.productId
        };
    }
    async verifyIdToken(authToken, req, flag, hasAbility) {
        if (!authToken?.info?.token) {
            (0, cm_libs_context_aware_error_1.throwError)(cm_libs_context_aware_error_1.errors.UNAUTHORISED, 'Id token is missing');
        }
        const idToken = await this.callVerifyIdToken(authToken, flag, hasAbility);
        authToken.type = auth_type_1.AuthTokenType.Bearer;
        authToken.grant = true;
        authToken.info.auth = { rules: undefined, ...idToken, ability: undefined };
        authToken.info.requestedBy = idToken.userId;
        authToken.info.requestedByName = idToken.userLogin;
        const ability = (0, auth_type_1.generateAbility)(idToken.rules, { req, accessInfo: authToken.info });
        authToken.info.ability = ability;
        authToken.info.auth.ability = ability;
        return authToken;
    }
    async callVerifyIdToken(authToken, flag, hasAbility) {
        // Decode token to get token details for validation
        if (!this.fnVerify || !flag) {
            const claim = auth_id_token_1.AuthIdToken.decode(authToken.info.token.replace('Bearer ', ''));
            // NOTE: Service id token claim should NOT include orgId, appDomain, and productId
            if (!claim ||
                (claim.orgId && this.filter.orgId !== claim.orgId) ||
                (claim.appDomain && this.filter.appDomain !== claim.appDomain) ||
                (claim.productId && this.filter.productId !== claim.productId)) {
                (0, cm_libs_context_aware_error_1.throwError)(cm_libs_context_aware_error_1.errors.UNAUTHORISED, 'Id token claim is out of scope', { claim, expected: this.filter });
            }
        }
        // Internal use by Identity service to verify idToken
        if (this.fnVerify) {
            // flag=true, identity service verifyIdToken endpoint
            const filter = flag ? authToken.authMetadata?.bearer || (0, cm_libs_context_aware_error_1.throwError)(cm_libs_context_aware_error_1.errors.UNAUTHORISED, 'Id token header metadata is missing') : { ...this.filter, hasAbility };
            return this.validateIdToken(await this.fnVerify(authToken, { ...filter, roleType: authToken.info.headers.roleType }));
        }
        const query = (0, graphql_tag_1.gql) `
            query {
                verifyIdToken {
                    orgId
                    appDomain
                    productId
                    userId
                    userLogin
                    localeCode
                    rules
                    rolesMetadata
                }
            }
        `;
        const body = { query: (0, graphql_1.print)(query) };
        const headers = {
            authorization: authToken.info.token,
            'x-org-id': this.filter.orgId,
            'x-app-domain': this.filter.appDomain,
            'x-product-id': this.filter.productId
        };
        if (undefined !== authToken.info.headers?.roleType) {
            headers['x-role-type'] = authToken.info.headers?.roleType;
        }
        if (undefined !== hasAbility) {
            headers['x-has-ability'] = JSON.stringify(hasAbility);
        }
        if (undefined !== authToken.info.headers?.metadata) {
            headers['x-metadata'] = JSON.stringify(authToken.info.headers?.metadata);
        }
        // Cache is not activated when it is not configured or MFA is required
        if (0 === this.cacheMax || authToken.info.headers?.metadata?.mfa) {
            // Call Identity service to verify idToken, check blacklist, and return idToken info (e.g. rules, etc)
            return this.validateIdToken(await (0, intercall_1.callGraphQl)(this.identityUrl, body, 'verifyIdToken', headers));
        }
        const cacheKey = crypto_1.default.createHash('sha256').update(JSON.stringify(headers)).end().setEncoding('hex').read();
        const cache = this.caches[cacheKey];
        if (cache) {
            return cache.res;
        }
        // Call Identity service to verify idToken, check blacklist, and return idToken info (e.g. rules, etc)
        const res = this.validateIdToken(await (0, intercall_1.callGraphQl)(this.identityUrl, body, 'verifyIdToken', headers));
        if (res && this.cacheMax > Object.keys(this.caches).length) {
            this.caches[cacheKey] = {
                res,
                expiryTimer: setTimeout(() => {
                    delete this.caches[cacheKey];
                }, this.cacheDuration)
            };
        }
        return res;
    }
    validateIdToken(idToken) {
        if (!idToken) {
            (0, cm_libs_context_aware_error_1.throwError)(cm_libs_context_aware_error_1.errors.INVALID_FORMAT, 'Id token is missing', { function: 'validateIdToken' });
        }
        return (0, type_1.validateOrThrowError)(joi_1.default.object().keys(auth_type_verify_1.schemaVerifyIdToken), idToken, { stripUnknown: true }, 'validateIdToken');
    }
}
exports.AuthBearer = AuthBearer;
//# sourceMappingURL=auth-bearer.js.map