"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DomainEventProducer = exports.initDomainEventProducer = void 0;
const nanoid_1 = require("nanoid");
const cm_libs_kafka_1 = require("cm-libs-kafka");
const cm_libs_utils_1 = require("cm-libs-utils");
const config_base_1 = require("../config-base");
const domain_event_1 = require("./domain-event");
const domain_event_cipher_1 = require("./domain-event-cipher");
function initDomainEventProducer(schema, metadataSchema, topics, cipher, disableSchemaRegistry) {
    const KAFKA_URL = config_base_1.ConfigBase.get('KAFKA_URL', (0, cm_libs_utils_1.notAllowNullOrEmpty)(cm_libs_utils_1.checkString));
    if (!cipher) {
        const KAFKA_KSN = config_base_1.ConfigBase.get('KAFKA_KSN', (0, cm_libs_utils_1.allowNullOrEmpty)(cm_libs_utils_1.checkString));
        if (KAFKA_KSN) {
            const KAFKA_BDKSET = (0, domain_event_cipher_1.getBdkSet)('KAFKA_BDKSET', false);
            cipher = new domain_event_cipher_1.DomainEventCipher(KAFKA_BDKSET, Buffer.from(KAFKA_KSN, 'hex'), domain_event_cipher_1.DEFAULT_CIPHER_ALGO, domain_event_cipher_1.DEFAULT_IV_LENGTH);
        }
    }
    return new DomainEventProducer(KAFKA_URL, topics, schema, metadataSchema, cipher, disableSchemaRegistry);
}
exports.initDomainEventProducer = initDomainEventProducer;
class DomainEventProducer {
    config;
    topics;
    producer;
    domainEvent;
    constructor(config, topics, schema, metadataSchema, cipher, disableSchemaRegistry = false) {
        const kafkaConfig = 'string' === typeof config ? JSON.parse(config) : config;
        const KAFKA_SSL_CA = config_base_1.ConfigBase.get('KAFKA_SSL_CA', (0, cm_libs_utils_1.allowNullOrEmpty)(cm_libs_utils_1.checkString));
        const KAFKA_SSL_CERT = config_base_1.ConfigBase.get('KAFKA_SSL_CERT', (0, cm_libs_utils_1.allowNullOrEmpty)(cm_libs_utils_1.checkString));
        const KAFKA_SSL_KEY = config_base_1.ConfigBase.get('KAFKA_SSL_KEY', (0, cm_libs_utils_1.allowNullOrEmpty)(cm_libs_utils_1.checkString));
        if (KAFKA_SSL_CA || KAFKA_SSL_CERT || KAFKA_SSL_KEY) {
            const ssl = 'boolean' === typeof kafkaConfig.ssl ? undefined : kafkaConfig.ssl;
            const ca = KAFKA_SSL_CA ? KAFKA_SSL_CA : ssl?.ca;
            const cert = KAFKA_SSL_CERT ? KAFKA_SSL_CERT : ssl?.cert;
            const key = KAFKA_SSL_KEY ? KAFKA_SSL_KEY : ssl?.key;
            kafkaConfig.ssl = { ...ssl, ca, cert, key };
        }
        this.config = kafkaConfig;
        this.topics = topics || [];
        this.domainEvent = new domain_event_1.DomainEvent(schema, metadataSchema, cipher, disableSchemaRegistry);
    }
    async term() {
        if (this.producer) {
            await this.producer.term();
            this.producer = undefined;
        }
    }
    async publish(message, topic) {
        message.eventType = message.eventType || null;
        message.eventId = message.eventId || (0, nanoid_1.nanoid)();
        message.correlationId = message.correlationId || (0, nanoid_1.nanoid)();
        message.timestamp = message.timestamp || (0, cm_libs_utils_1.timeNow)();
        const encodedMessage = await this.domainEvent.encode(message);
        const producer = await this.getProducer();
        const topics = topic ? this.topics.concat([topic]) : this.topics;
        if (1 === topics.length) {
            const result = await producer.send(topics[0], encodedMessage, message.eventId);
            return [result];
        }
        const batch = {
            topicMessages: []
        };
        for (const topic of topics) {
            batch.topicMessages.push({
                topic,
                messages: [
                    {
                        value: encodedMessage,
                        key: message.eventId
                    }
                ]
            });
        }
        return producer.sendBatch(batch);
    }
    async publishRaw(encodedMessage, messageKey, topic) {
        const producer = await this.getProducer();
        const topics = topic ? this.topics.concat([topic]) : this.topics;
        if (1 === topics.length) {
            const result = await producer.send(topics[0], encodedMessage, messageKey);
            return [result];
        }
        const batch = {
            topicMessages: []
        };
        for (const topic of topics) {
            batch.topicMessages.push({
                topic,
                messages: [
                    {
                        value: encodedMessage,
                        key: messageKey
                    }
                ]
            });
        }
        return producer.sendBatch(batch);
    }
    async getProducer() {
        if (!this.producer) {
            this.producer = await cm_libs_kafka_1.KafkaProducer.init(this.config);
        }
        return this.producer;
    }
    get ready() {
        if (this.producer) {
            return this.producer.ready;
        }
        return true;
    }
    get schemaRegistryId() {
        return this.domainEvent.schemaRegistryId;
    }
    get metadataSchemaRegistryId() {
        return this.domainEvent.metadataSchemaRegistryId;
    }
}
exports.DomainEventProducer = DomainEventProducer;
//# sourceMappingURL=domain-event-producer.js.map