"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Registry = exports.DomainSchema = void 0;
const avsc_1 = __importDefault(require("avsc"));
const confluent_schema_registry_1 = require("@kafkajs/confluent-schema-registry");
const schema_registry_sdk_1 = require("@alicloud/schema-registry-sdk");
const cm_libs_context_aware_error_1 = require("cm-libs-context-aware-error");
class DomainSchema {
    schemaValue;
    schema;
    registry;
    _registryId;
    constructor(schema, disableSchemaRegistry = false) {
        if ('string' !== typeof schema) {
            schema = JSON.stringify(schema);
        }
        this.schemaValue = schema;
        if ('boolean' === typeof disableSchemaRegistry) {
            if (!disableSchemaRegistry) {
                this.registry = Registry.isEnabled ? Registry.instance : undefined;
            }
        }
        else {
            this.registry = disableSchemaRegistry || (Registry.isEnabled ? Registry.instance : undefined);
        }
        if (!this.registry && schema) {
            this.schema = avsc_1.default.Type.forSchema(JSON.parse(schema));
        }
    }
    async encode(value) {
        try {
            if (this.schema) {
                return this.schema.toBuffer(value);
            }
            if (undefined === this.registryId) {
                if (!this.isEnabled) {
                    (0, cm_libs_context_aware_error_1.throwError)(cm_libs_context_aware_error_1.errors.NOT_INITIALIZED, 'Schema is not initialized');
                }
                this._registryId = await this.registry.register(this.schemaValue);
            }
            return await this.registry.encode(this.registryId, value);
        }
        catch (error) {
            (0, cm_libs_context_aware_error_1.throwFromError)(error, cm_libs_context_aware_error_1.errors.INVALID_MESSAGE, { registryId: this.registryId });
        }
    }
    async decode(value) {
        try {
            if (this.schema) {
                return this.schema.fromBuffer(value);
            }
            return await this.registry.decode(value);
        }
        catch (error) {
            (0, cm_libs_context_aware_error_1.throwFromError)(error, cm_libs_context_aware_error_1.errors.INVALID_MESSAGE);
        }
    }
    get isEnabled() {
        return this.schemaValue ? true : false;
    }
    get isSchemaRegistryEnabled() {
        return this.registry ? true : false;
    }
    get registryId() {
        return this._registryId;
    }
}
exports.DomainSchema = DomainSchema;
class Registry {
    static self;
    url;
    overrides;
    schemaRegistry;
    alicloudSchemaRegistry;
    constructor(url, overrides) {
        this.url = url;
        this.overrides = overrides;
        if (overrides?.accessKeyId && overrides?.accessKeySecret && overrides?.groupId) {
            this.alicloudSchemaRegistry = new schema_registry_sdk_1.SchemaRegistry({ endpoint: url, ...overrides });
        }
        else {
            this.schemaRegistry = new confluent_schema_registry_1.SchemaRegistry({ host: url, ...overrides });
        }
    }
    static init(url, overrides) {
        this.self = new Registry(url, overrides);
        return this.self;
    }
    static term() {
        this.self = undefined;
    }
    clone() {
        return new Registry(this.url, this.overrides);
    }
    async register(schema) {
        if (this.alicloudSchemaRegistry) {
            return (await this.alicloudSchemaRegistry.register({ schema, type: schema_registry_sdk_1.SchemaType.AVRO }))?.id;
        }
        return (await this.schemaRegistry.register({ schema, type: confluent_schema_registry_1.SchemaType.AVRO }))?.id;
    }
    async encode(id, payload) {
        if (this.alicloudSchemaRegistry) {
            return await this.alicloudSchemaRegistry.encode(id, payload);
        }
        return this.schemaRegistry.encode(id, payload);
    }
    async decode(payload) {
        if (this.alicloudSchemaRegistry) {
            return this.alicloudSchemaRegistry.decode(payload);
        }
        return this.schemaRegistry.decode(payload);
    }
    static get instance() {
        return this.self || (0, cm_libs_context_aware_error_1.throwError)(cm_libs_context_aware_error_1.errors.NOT_INITIALIZED, 'Registry initialization is required');
    }
    static get isEnabled() {
        return undefined !== this.self;
    }
}
exports.Registry = Registry;
//# sourceMappingURL=domain-schema.js.map