"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.callRest = exports.callGraphQl = void 0;
const axios_1 = __importDefault(require("axios"));
const cm_libs_context_aware_error_1 = require("cm-libs-context-aware-error");
const cm_libs_context_aware_logger_1 = require("cm-libs-context-aware-logger");
const cm_libs_utils_1 = require("cm-libs-utils");
const devMode = 'production' !== process.env.NODE_ENV;
async function callGraphQl(url, query, fn, headers, config) {
    try {
        if (devMode) {
            (0, cm_libs_context_aware_logger_1.log)(cm_libs_context_aware_logger_1.Level.CRITICAL, () => {
                const reqHeaders = headers ? JSON.stringify(headers, null, 2) : '';
                return `TRACE CALL_GRAPHQL REQ: ${url} - ${fn}\nHEADERS: ${reqHeaders}\nBODY: ${JSON.stringify(query, null, 2)}`;
            });
        }
        else {
            (0, cm_libs_context_aware_logger_1.log)(cm_libs_context_aware_logger_1.Level.DEBUG, () => `CALL_GRAPHQL REQ: ${url} - ${fn}`);
        }
        const reqConfig = defaultConfig({ headers, ...config });
        const start = (0, cm_libs_utils_1.timeNow)();
        const res = await axios_1.default.post(url, query, reqConfig);
        const body = res?.data;
        if (devMode) {
            (0, cm_libs_context_aware_logger_1.log)(cm_libs_context_aware_logger_1.Level.CRITICAL, () => {
                const elapsed = (0, cm_libs_utils_1.timeElapsed)(start).toString();
                return `TRACE CALL_GRAPHQL RES: ${url} - ${fn}, elapsed: ${elapsed + ' ms'}\nHEADERS: ${JSON.stringify(res?.headers, null, 2)}\nBODY: ${JSON.stringify(body, null, 2)}`;
            });
        }
        else {
            (0, cm_libs_context_aware_logger_1.log)(cm_libs_context_aware_logger_1.Level.DEBUG, () => {
                const elapsed = (0, cm_libs_utils_1.timeElapsed)(start).toString();
                return `CALL_GRAPHQL RES: ${url} - ${fn}, elapsed: ${elapsed + ' ms'}`;
            });
        }
        // Graphql endpoint always return 200, including error returned
        if (body?.errors) {
            const error = body.errors[0];
            if (error) {
                (0, cm_libs_context_aware_error_1.throwFromError)(error, error.extensions?.code || cm_libs_context_aware_error_1.errors.INTERCALL_ERROR, { url, query, fn, error }, true);
            }
            (0, cm_libs_context_aware_error_1.throwError)(cm_libs_context_aware_error_1.errors.INTERCALL_ERROR, 'callGraphQl ERROR', { url, query, fn, body }, true);
        }
        return body?.data?.[fn];
    }
    catch (error) {
        // Most likely this error is due to network or server resources
        (0, cm_libs_context_aware_error_1.throwFromError)(error, cm_libs_context_aware_error_1.errors.INTERCALL_ERROR, { url, query, fn, status: error?.response?.status }, true);
    }
}
exports.callGraphQl = callGraphQl;
async function callRest(url, method, params, headers, config) {
    try {
        if (devMode) {
            (0, cm_libs_context_aware_logger_1.log)(cm_libs_context_aware_logger_1.Level.CRITICAL, () => {
                const reqHeaders = headers ? JSON.stringify(headers, null, 2) : '';
                const reqBody = params ? JSON.stringify(params, null, 2) : '';
                return `TRACE CALL_REST REQ: ${method} ${url}\nHEADERS: ${reqHeaders}\nBODY: ${reqBody}`;
            });
        }
        else {
            (0, cm_libs_context_aware_logger_1.log)(cm_libs_context_aware_logger_1.Level.DEBUG, () => `CALL_REST REQ: ${method} ${url}`);
        }
        const reqConfig = defaultConfig({ url, method, headers, data: params, ...config });
        const start = (0, cm_libs_utils_1.timeNow)();
        const res = await (0, axios_1.default)(reqConfig);
        const body = res?.data;
        if (devMode) {
            (0, cm_libs_context_aware_logger_1.log)(cm_libs_context_aware_logger_1.Level.CRITICAL, () => {
                const elapsed = (0, cm_libs_utils_1.timeElapsed)(start).toString();
                return `TRACE CALL_REST RES: ${method} ${url}, elapsed: ${elapsed + ' ms'}\nHEADERS: ${JSON.stringify(res?.headers, null, 2)}\nBODY: ${JSON.stringify(body, null, 2)}`;
            });
        }
        else {
            (0, cm_libs_context_aware_logger_1.log)(cm_libs_context_aware_logger_1.Level.DEBUG, () => {
                const elapsed = (0, cm_libs_utils_1.timeElapsed)(start).toString();
                return `CALL_REST RES: ${method} ${url}, elapsed: ${elapsed + ' ms'}`;
            });
        }
        // Rest endpoint always return 200, including error returned
        if (body?.errors) {
            const error = body?.errors[0];
            if (error) {
                (0, cm_libs_context_aware_error_1.throwFromError)(error, error?.extensions?.code || cm_libs_context_aware_error_1.errors.INTERCALL_ERROR, { url, data: params, method, error }, true);
            }
            (0, cm_libs_context_aware_error_1.throwError)(cm_libs_context_aware_error_1.errors.INTERCALL_ERROR, 'CALL_REST ERROR', { url, data: params, method, body }, true);
        }
        return (body?.data || body);
    }
    catch (error) {
        (0, cm_libs_context_aware_error_1.throwFromError)(error, cm_libs_context_aware_error_1.errors.INTERCALL_ERROR, { url, data: params, method, status: error?.status }, true);
    }
}
exports.callRest = callRest;
function defaultConfig(overrides) {
    const config = {
        maxBodyLength: Infinity,
        maxContentLength: Infinity,
        ...overrides
    };
    if ((0, cm_libs_utils_1.isUndefinedOrNullOrEmpty)(config.timeout)) {
        config.timeout = 120000; // Default timeout is 120 seconds
    }
    return config;
}
//# sourceMappingURL=intercall.js.map