"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Permission = exports.Role = exports.BankRole = exports.PAIDChainRole = exports.OrganizationAllow = void 0;
const cm_libs_context_aware_error_1 = require("cm-libs-context-aware-error");
const cm_libs_utils_1 = require("cm-libs-utils");
const lodash_1 = __importDefault(require("lodash"));
const config_base_1 = require("../config-base");
const validator_1 = require("../validator");
var OrganizationAllow;
(function (OrganizationAllow) {
    OrganizationAllow["PAIDChain"] = "PAIDChain";
    OrganizationAllow["BANK"] = "BANK";
    OrganizationAllow["APPLICANT"] = "APPLICANT";
})(OrganizationAllow = exports.OrganizationAllow || (exports.OrganizationAllow = {}));
var PAIDChainRole;
(function (PAIDChainRole) {
    PAIDChainRole["MAKER"] = "MAKER";
    PAIDChainRole["CHECKER"] = "CHECKER";
    PAIDChainRole["APPROVER"] = "APPROVER";
    PAIDChainRole["OFFICER"] = "OFFICER";
    PAIDChainRole["MCH_MERCHANT_API"] = "MCH_MERCHANT_API";
})(PAIDChainRole = exports.PAIDChainRole || (exports.PAIDChainRole = {}));
var BankRole;
(function (BankRole) {
    BankRole["MAKER"] = "MAKER";
    BankRole["CHECKER"] = "CHECKER";
    BankRole["APPROVER"] = "APPROVER";
    BankRole["OFFICER"] = "OFFICER";
})(BankRole = exports.BankRole || (exports.BankRole = {}));
var Role;
(function (Role) {
    Role["APPLICANT"] = "APPLICANT";
})(Role = exports.Role || (exports.Role = {}));
class Permission {
    static PAIDChainOrgCode = 'OMPHALOS_GENESIS:RAPID_MERCHANT_ONBOARDING';
    static BankOrgCodePrefix = 'OMPHALOS_GENESIS:RAPID_MERCHANT_ONBOARDING:BANK_';
    static IS_BANK_INFRA = config_base_1.ConfigBase.get('IS_BANK_INFRA', (0, cm_libs_utils_1.allowNullOrEmpty)(cm_libs_utils_1.checkBoolean, false));
    static DISABLE_AUTH_CHECK = config_base_1.ConfigBase.get('DISABLE_AUTH_CHECK', (0, cm_libs_utils_1.allowNullOrEmpty)(cm_libs_utils_1.checkBoolean, false));
    static check(organizationAllows, accessInfo) {
        organizationAllows = Array.isArray(organizationAllows) ? organizationAllows : [organizationAllows];
        if (this.DISABLE_AUTH_CHECK)
            return undefined;
        const { roleOrgCode, roleType } = this.details(accessInfo);
        if (!roleOrgCode)
            (0, cm_libs_context_aware_error_1.throwError)(cm_libs_context_aware_error_1.MerchantErrorCodes.PermissionError, 'No OrgCode');
        if (!roleType)
            (0, cm_libs_context_aware_error_1.throwError)(cm_libs_context_aware_error_1.MerchantErrorCodes.PermissionError, 'No RoleType');
        const isApplicant = this.ApplicationCheck({ roleOrgCode, roleType, organizationAllows });
        if (isApplicant)
            return isApplicant;
        const isPAIDChain = this.PAIDChainCheck({ roleOrgCode, roleType, organizationAllows });
        if (isPAIDChain)
            return isPAIDChain;
        const isBank = this.BankCheck({ roleOrgCode, roleType, organizationAllows });
        if (isBank)
            return isBank;
        (0, cm_libs_context_aware_error_1.throwError)(cm_libs_context_aware_error_1.MerchantErrorCodes.PermissionError, 'No authority', { details: { roleOrgCode, roleType, organizationAllows: JSON.stringify(organizationAllows) } });
    }
    static ApplicationCheck({ organizationAllows, roleOrgCode, roleType }) {
        if (roleOrgCode === this.PAIDChainOrgCode && organizationAllows.includes(OrganizationAllow.APPLICANT)) {
            return Role[roleType];
        }
    }
    static PAIDChainCheck({ organizationAllows, roleOrgCode, roleType }) {
        if (roleOrgCode === this.PAIDChainOrgCode && organizationAllows.includes(OrganizationAllow.PAIDChain)) {
            return PAIDChainRole[roleType];
        }
    }
    static BankCheck({ organizationAllows, roleOrgCode, roleType }) {
        if (lodash_1.default.startsWith(roleOrgCode, this.BankOrgCodePrefix) && organizationAllows.includes(OrganizationAllow.BANK)) {
            return BankRole[roleType];
        }
    }
    static async executeFunctionWithRole(actual, expected, args) {
        args = Array.isArray(args) ? args : [args];
        if (Permission.DISABLE_AUTH_CHECK) {
            const functionArgs = args[0];
            return await (0, validator_1.baseHandler)(functionArgs.thisObject, functionArgs.fn, functionArgs.schema, lodash_1.default.cloneDeep(functionArgs.params), lodash_1.default.cloneDeep(functionArgs.accessInfo));
        }
        else {
            expected = Array.isArray(expected) ? expected : [expected];
            return await this.loopEveryRoleAndCheck(actual, expected, args);
        }
    }
    static async loopEveryRoleAndCheck(actual, expected, args) {
        for (let index = 0; index < expected.length; index++) {
            const element = expected[index];
            if (actual === element) {
                const functionArgs = args[index] || args[0];
                return await (0, validator_1.baseHandler)(functionArgs.thisObject, functionArgs.fn, functionArgs.schema, lodash_1.default.cloneDeep(functionArgs.params), lodash_1.default.cloneDeep(functionArgs.accessInfo));
            }
        }
        (0, cm_libs_context_aware_error_1.throwError)(cm_libs_context_aware_error_1.MerchantErrorCodes.PermissionError, 'Permission not right', { expected, actual });
    }
    static isBank() {
        return this.IS_BANK_INFRA;
    }
    static details(accessInfo) {
        const { roleType, roleOrgCode, roleOrgId } = accessInfo?.auth?.rolesMetadata?.[0]?.attributes || {};
        return {
            roleType,
            roleOrgCode,
            roleOrgId,
            remoteIp: accessInfo?.headers?.remoteIp,
            isBankOrg: roleOrgCode ? roleOrgCode.startsWith(Permission.BankOrgCodePrefix) : false,
            isPAIDChainOrg: roleOrgCode === this.PAIDChainOrgCode
        };
    }
}
exports.Permission = Permission;
//# sourceMappingURL=permission.js.map