"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.BaseModelMiddlewareRoleAbilityLog = void 0;
const nanoid_1 = require("nanoid");
const graphql_1 = require("graphql");
const graphql_tag_1 = require("graphql-tag");
const cm_libs_context_aware_error_1 = require("cm-libs-context-aware-error");
const cm_libs_context_aware_logger_1 = require("cm-libs-context-aware-logger");
const intercall_1 = require("../intercall");
class BaseModelMiddlewareRoleAbilityLog {
    static self;
    devMode;
    idToken;
    identityUrl;
    disableLogMessage;
    constructor(devMode, idToken, identityUrl, disableLogMessage) {
        this.devMode = devMode;
        this.idToken = idToken;
        this.identityUrl = identityUrl;
        this.disableLogMessage = disableLogMessage;
    }
    static init(devMode, idToken, identityUrl, disableLogMessage) {
        this.self = new BaseModelMiddlewareRoleAbilityLog(devMode, idToken, identityUrl, disableLogMessage);
        return this.self;
    }
    static get isEnabled() {
        return this.self && this.self.devMode && this.self.idToken && this.self.identityUrl ? true : false;
    }
    static get instance() {
        return this.isEnabled ? this.self : (0, cm_libs_context_aware_error_1.throwError)(cm_libs_context_aware_error_1.errors.NOT_INITIALIZED, 'BaseModelMiddlewareRoleAbilityLog initialization is required');
    }
    async process(params, next, asyncLocalStorage) {
        try {
            const context = asyncLocalStorage.getStore();
            if (!context.disableRestriction) {
                if (!this.disableLogMessage) {
                    (0, cm_libs_context_aware_logger_1.log)(cm_libs_context_aware_logger_1.Level.DEBUG, () => `LOG ROLE ABILITY MODEL ${params.model}.${params.action}: ${JSON.stringify(params, null, 2)}`);
                }
                const logs = this.track(params.args, true, params.model, params.action);
                const correlationId = (0, nanoid_1.nanoid)();
                let seq = 0;
                const promises = [];
                for (const log of logs.reverse()) {
                    promises.push(this.callRoleModelAbilityLog(correlationId, log.root, seq++, log.model, log.action, log.args, context));
                }
                await Promise.all(promises);
                if (!this.disableLogMessage) {
                    (0, cm_libs_context_aware_logger_1.log)(cm_libs_context_aware_logger_1.Level.DETAIL, () => `LOGGED ROLE ABILITY MODEL ${params.model}.${params.action}: ${JSON.stringify(logs, null, 2)}`);
                }
            }
        }
        catch (error) {
            (0, cm_libs_context_aware_logger_1.log)(cm_libs_context_aware_logger_1.Level.WARNING, () => {
                const { code, reason } = (0, cm_libs_context_aware_error_1.describeError)(error);
                return `LOG ROLE ABILITY MODEL WARNING ${params?.model}:${params?.action} - ${code}: ${reason}`;
            });
        }
        return next(params);
    }
    track(args, root, model, action) {
        let logs = [];
        if (!action) {
            if (args?.data) {
                const multiple = Array.isArray(args?.data);
                if (args.where) {
                    action = multiple ? 'updateMany' : 'update';
                }
                else {
                    action = multiple ? 'createMany' : 'create';
                }
            }
            else if (args?.create && args?.update && args?.where) {
                action = 'upsert';
            }
            else if (args.select) {
                action = 'findMany';
            }
        }
        switch (action) {
            case 'findUnique':
            case 'findMany':
                logs = logs.concat(logs, this.trackFind(args, root, model, action));
                break;
            case 'create':
            case 'createMany':
                logs = logs.concat(logs, this.trackCreate(args, root, model, action));
                break;
            case 'update':
            case 'updateMany':
                logs = logs.concat(logs, this.trackUpdate(args, root, model, action));
                break;
            case 'upsert':
                logs = logs.concat(logs, this.trackUpsert(args, root, model, action));
                break;
            case 'delete':
            case 'deleteMany':
                logs = logs.concat(logs, this.trackDelete(args, root, model, action));
                break;
            case 'count':
                break;
            default:
                if (undefined !== action && !this.disableLogMessage) {
                    (0, cm_libs_context_aware_logger_1.log)(cm_libs_context_aware_logger_1.Level.WARNING, () => `POSSIBLE UNTRACKED ROLE ABILITY MODEL ${model}.${action}: ${JSON.stringify(args, null, 2)}`);
                }
        }
        return logs;
    }
    trackUpsert(args, root, model, action) {
        let logs = [];
        if (args?.update && args?.create && args?.where) {
            for (const subModelName of Object.keys(args.create)) {
                const subModel = args.create[subModelName];
                if ('object' === typeof subModel) {
                    logs = logs.concat(logs, this.track(subModel, false, subModelName));
                }
            }
            for (const subModelName of Object.keys(args.update)) {
                const subModel = args.update[subModelName];
                if ('object' === typeof subModel) {
                    logs = logs.concat(logs, this.track(subModel, false, subModelName));
                }
            }
            logs.push({ root, action, model, args });
        }
        return logs;
    }
    trackFind(args, root, model, action) {
        let logs = [];
        if (args?.select) {
            for (const subModelName of Object.keys(args.select)) {
                const subModel = args.select[subModelName];
                if ('object' === typeof subModel) {
                    logs = logs.concat(logs, this.track(subModel, false, subModelName));
                }
            }
            logs.push({ root, action, model, args });
        }
        return logs;
    }
    trackCreate(args, root, model, action) {
        let logs = [];
        const create = args?.create || args?.connectOrCreate || args?.data;
        if (undefined !== create) {
            for (const data of Array.isArray(create) ? create : [create]) {
                for (const subModelName of Object.keys(data)) {
                    const subModel = data[subModelName];
                    if ('object' === typeof subModel) {
                        const subCreate = subModel.connectOrCreate || subModel.create;
                        if (undefined !== subCreate) {
                            const action = Array.isArray(subCreate) ? 'createMany' : 'create';
                            logs = logs.concat(logs, this.trackCreate(subModel, false, subModelName, action));
                        }
                        else {
                            logs = logs.concat(logs, this.track(subModel, false, subModelName));
                        }
                    }
                }
            }
            logs.push({ root, action, model, args });
        }
        return logs;
    }
    trackUpdate(args, root, model, action) {
        let logs = [];
        const data = args?.data || args?.update;
        if (data && args?.where) {
            for (const subData of Array.isArray(data) ? data : [data]) {
                for (const subModelName of Object.keys(subData)) {
                    const subModel = subData[subModelName];
                    if ('object' === typeof subModel) {
                        logs = logs.concat(logs, this.track(subModel, false, subModelName));
                    }
                }
            }
            logs.push({ root, action, model, args });
        }
        return logs;
    }
    trackDelete(args, root, model, action) {
        return [{ root, action, model, args }];
    }
    async callRoleModelAbilityLog(correlationId, root, seq, model, action, args, context) {
        if (0 < context.accessInfo.auth?.productId && 0 < context.accessInfo.requestedBy) {
            const data = {
                correlationId,
                root,
                seq,
                action,
                model,
                args,
                product: {
                    connect: { id: context.accessInfo.auth?.productId }
                },
                user: {
                    connect: { id: context.accessInfo.requestedBy }
                },
                roleType: context.accessInfo.headers?.roleType
            };
            const mutation = (0, graphql_tag_1.gql) `
                mutation ($data: UserModelAbilityLogAddInput!) {
                    addUserModelAbilityLog(data: $data) {
                        id
                    }
                }
            `;
            const body = { query: (0, graphql_1.print)(mutation), variables: { data } };
            const headers = {
                authorization: `Bearer ${this.idToken}`
            };
            // Call Identity service to add role ability model log
            await (0, intercall_1.callGraphQl)(this.identityUrl, body, 'addUserModelAbilityLog', headers);
        }
    }
}
exports.BaseModelMiddlewareRoleAbilityLog = BaseModelMiddlewareRoleAbilityLog;
//# sourceMappingURL=model-middleware-role-ability-log.js.map