"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TcpBase = exports.Socket = void 0;
const net_1 = require("net");
Object.defineProperty(exports, "Socket", { enumerable: true, get: function () { return net_1.Socket; } });
class TcpBase {
    connections = {};
    _connectionCount = 0;
    async term() {
        for (const clientId of Object.keys(this.connections)) {
            const connection = this.connections[clientId];
            const socket = connection.tlsSocket || connection.socket;
            socket?.destroy();
            if (this.removeConnection(clientId)) {
                try {
                    await this.onClose(connection, undefined);
                }
                catch (error) {
                    // Ignore this error
                }
            }
        }
    }
    async addConnection(socket, tlsSocket, clientId, ref, keepAliveInterval, idleTimeout) {
        const connection = {
            clientId,
            socket,
            tlsSocket,
            buffer: Buffer.alloc(0),
            isIdle: true,
            ref
        };
        socket = tlsSocket || socket;
        if (0 < idleTimeout) {
            socket.setTimeout(idleTimeout, () => socket.destroy());
        }
        if (0 < keepAliveInterval) {
            socket.setKeepAlive(true, keepAliveInterval);
        }
        let raisedError;
        socket.on('close', () => {
            return this.onCloseEvent(connection, raisedError);
        });
        /* istanbul ignore next */
        socket.on('error', (error) => {
            raisedError = error;
            socket.destroy();
        });
        socket.on('data', async (data) => {
            return this.onDataEvent(connection, data);
        });
        this.connections[connection.clientId] = connection;
        this._connectionCount++;
        try {
            await this.onConnected(connection);
        }
        catch (error) {
            socket.destroy(error);
        }
        return connection;
    }
    removeConnection(clientId) {
        const connection = this.connections[clientId];
        if (connection) {
            delete this.connections[clientId];
            this._connectionCount--;
            return true;
        }
        return false;
    }
    async onDataEvent(connection, data) {
        try {
            connection.buffer = Buffer.concat([connection.buffer, data]);
            if (connection.isIdle) {
                connection.isIdle = false;
                do {
                    const usedLength = await this.onData(connection, data);
                    if (0 >= usedLength) {
                        break;
                    }
                    connection.buffer = connection.buffer.subarray(usedLength);
                    data = undefined;
                } while (0 < connection.buffer.length);
                connection.isIdle = true;
            }
        }
        catch (error) {
            const socket = connection.tlsSocket || connection.socket;
            socket.destroy(error);
        }
    }
    async onCloseEvent(connection, raisedError) {
        if (this.removeConnection(connection.clientId)) {
            try {
                await this.onClose(connection, raisedError);
            }
            catch (error) {
                // Ignore this error
            }
        }
    }
    get connectionCount() {
        return this._connectionCount;
    }
    static async socketWrite(socket, data) {
        if (!socket.write(data)) {
            return new Promise((resolve, reject) => {
                // Handle the socket buffer backpressure
                socket.once('drain', () => resolve()).once('error', (error) => reject(error));
            });
        }
    }
    static resetIdleTimeout(socket, idleTimeout) {
        if (0 <= idleTimeout) {
            socket.setTimeout(idleTimeout, () => socket.destroy());
        }
    }
}
exports.TcpBase = TcpBase;
//# sourceMappingURL=tcp-base.js.map